@@ -4,6 +4,7 @@ import (
 	"context"
 	"fmt"
 	"net/http"
+	"net/url"
 	"strings"
 
 	"github.com/MakeNowJust/heredoc"
@@ -44,18 +45,39 @@ func NewCmdClone(f *cmdutil.Factory, runF func(*CloneOptions) error) *cobra.Comm
 		Short: "Clone a repository locally",
 		Long: heredoc.Docf(`
 			Clone a GitHub repository locally. Pass additional %[1]sgit clone%[1]s flags by listing
-			them after "--".
+			them after %[1]s--%[1]s.
 
-			If the "OWNER/" portion of the "OWNER/REPO" repository argument is omitted, it
+			If the %[1]sOWNER/%[1]s portion of the %[1]sOWNER/REPO%[1]s repository argument is omitted, it
 			defaults to the name of the authenticating user.
 
+			When a protocol scheme is not provided in the repository argument, the %[1]sgit_protocol%[1]s will be
+			chosen from your configuration, which can be checked via %[1]sgh config get git_protocol%[1]s. If the protocol
+			scheme is provided, the repository will be cloned using the specified protocol.
+
 			If the repository is a fork, its parent repository will be added as an additional
-			git remote called "upstream". The remote name can be configured using %[1]s--upstream-remote-name%[1]s.
-			The %[1]s--upstream-remote-name%[1]s option supports an "@owner" value which will name
+			git remote called %[1]supstream%[1]s. The remote name can be configured using %[1]s--upstream-remote-name%[1]s.
+			The %[1]s--upstream-remote-name%[1]s option supports an %[1]s@owner%[1]s value which will name
 			the remote after the owner of the parent repository.
 
 			If the repository is a fork, its parent repository will be set as the default remote repository.
 		`, "`"),
+		Example: heredoc.Doc(`
+			# Clone a repository from a specific org
+			$ gh repo clone cli/cli
+
+			# Clone a repository from your own account
+			$ gh repo clone myrepo
+
+			# Clone a repo, overriding git protocol configuration
+			$ gh repo clone https://github.com/cli/cli
+			$ gh repo clone git@github.com:cli/cli.git
+
+			# Clone a repository to a custom directory
+			$ gh repo clone cli/cli workspace/cli
+
+			# Clone a repository with additional git clone flags
+			$ gh repo clone cli/cli -- --depth=1
+		`),
 		RunE: func(cmd *cobra.Command, args []string) error {
 			opts.Repository = args[0]
 			opts.GitArgs = args[1:]
@@ -99,10 +121,12 @@ func cloneRun(opts *CloneOptions) error {
 	var protocol string
 
 	if repositoryIsURL {
-		repoURL, err := git.ParseURL(opts.Repository)
+		initialURL, err := git.ParseURL(opts.Repository)
 		if err != nil {
 			return err
 		}
+
+		repoURL := simplifyURL(initialURL)
 		repo, err = ghrepo.FromURL(repoURL)
 		if err != nil {
 			return err
@@ -129,10 +153,7 @@ func cloneRun(opts *CloneOptions) error {
 			return err
 		}
 
-		protocol, err = cfg.GetOrDefault(repo.RepoHost(), "git_protocol")
-		if err != nil {
-			return err
-		}
+		protocol = cfg.GitProtocol(repo.RepoHost())
 	}
 
 	wantsWiki := strings.HasSuffix(repo.RepoName(), ".wiki")
@@ -166,10 +187,7 @@ func cloneRun(opts *CloneOptions) error {
 
 	// If the repo is a fork, add the parent as an upstream remote and set the parent as the default repo.
 	if canonicalRepo.Parent != nil {
-		protocol, err := cfg.GetOrDefault(canonicalRepo.Parent.RepoHost(), "git_protocol")
-		if err != nil {
-			return err
-		}
+		protocol := cfg.GitProtocol(canonicalRepo.Parent.RepoHost())
 		upstreamURL := ghrepo.FormatRemoteURL(canonicalRepo.Parent, protocol)
 
 		upstreamName := opts.UpstreamName
@@ -204,3 +222,30 @@ func cloneRun(opts *CloneOptions) error {
 	}
 	return nil
 }
+
+// simplifyURL strips given URL of extra parts like extra path segments (i.e.,
+// anything beyond `/owner/repo`), query strings, or fragments. This function
+// never returns an error.
+//
+// The rationale behind this function is to let users clone a repo with any
+// URL related to the repo; like:
+//   - (Tree)              github.com/owner/repo/blob/main/foo/bar
+//   - (Deep-link to line) github.com/owner/repo/blob/main/foo/bar#L168
+//   - (Issue/PR comment)  github.com/owner/repo/pull/999#issue-9999999999
+//   - (Commit history)    github.com/owner/repo/commits/main/?author=foo
+func simplifyURL(u *url.URL) *url.URL {
+	result := &url.URL{
+		Scheme: u.Scheme,
+		User:   u.User,
+		Host:   u.Host,
+		Path:   u.Path,
+	}
+
+	pathParts := strings.SplitN(strings.Trim(u.Path, "/"), "/", 3)
+	if len(pathParts) <= 2 {
+		return result
+	}
+
+	result.Path = strings.Join(pathParts[0:2], "/")
+	return result
+}
